@@ -20,13 +20,10 @@
  */
 
 #include "sysincludes.h"
-#include "msdos.h"
-#include "vfat.h"
 #include "mtools.h"
 #include "file.h"
 #include "mainloop.h"
 #include "fs.h"
-#include "codepage.h"
 #include "file_name.h"
 
 #ifdef TEST_SIZE
@@ -65,7 +62,7 @@ static char mdir_longname[4*MAX_VNAMELEN+1];
 /*
  * Print an MSDOS directory date stamp.
  */
-static __inline__ void print_date(struct directory *dir)
+static inline void print_date(struct directory *dir)
 {
 	char year[5];
 	char day[3];
@@ -101,7 +98,7 @@ static __inline__ void print_date(struct directory *dir)
 /*
  * Print an MSDOS directory time stamp.
  */
-static __inline__ void print_time(struct directory *dir)
+static inline void print_time(struct directory *dir)
 {
 	char am_pm;
 	int hour = DOS_HOUR(dir);
@@ -140,7 +137,7 @@ static const char *dotted_num(mt_off_t num, size_t width, char **buf)
 	 * length is not exceeded (in %6ld, the result can be longer than 6!)
 	 */
 
-	numlo = num % 1000000000;
+	numlo = (unsigned long) (num % 1000000000);
 	numhi = (unsigned long) (num / 1000000000);
 
 	if(numhi && size > 9) {
@@ -159,7 +156,7 @@ static const char *dotted_num(mt_off_t num, size_t width, char **buf)
 	srcp = (*buf)+len;
 	dstp = (*buf)+len+1;
 
-	for ( ; dstp >= (*buf)+4 && isdigit (srcp[-1]); ) {
+	for ( ; dstp >= (*buf)+4 && isdigit ((unsigned char)srcp[-1]); ) {
 		srcp -= 3;  /* from here we copy three digits */
 		dstp -= 4;  /* that's where we put these 3 digits */
 	}
@@ -180,11 +177,11 @@ static const char *dotted_num(mt_off_t num, size_t width, char **buf)
 	return (*buf) + len-width;
 }
 
-static __inline__ int print_volume_label(Stream_t *Dir, char drive)
+static inline int print_volume_label(Stream_t *Dir, char drive)
 {
 	Stream_t *Stream = GetFs(Dir);
 	direntry_t entry;
-	DeclareThis(FsPublic_t);
+	DeclareThis(Fs_t);
 	char shortname[13];
 	char longname[VBUFSIZE];
 	int r;
@@ -210,10 +207,12 @@ static __inline__ int print_volume_label(Stream_t *Dir, char drive)
 	else
 		printf(" Volume in drive %c is %s",
 		       drive, shortname);
-	if(This->serialized)
+	if(getSerialized(This)) {
+		unsigned long serial_number = getSerialNumber(This);
 		printf("\n Volume Serial Number is %04lX-%04lX",
-		       (This->serial_number >> 16) & 0xffff,
-		       This->serial_number & 0xffff);
+		       (serial_number >> 16) & 0xffff,
+		       serial_number & 0xffff);
+	}
 	return 0;
 }
 
@@ -483,7 +482,7 @@ static int list_non_recurs_directory(direntry_t *entry, MainParam_t *mp)
 
 
 static int list_recurs_directory(direntry_t *entry UNUSEDP,
-				 MainParam_t *mp UNUSEDP)
+				 MainParam_t *mp)
 {
 	MainParam_t subMp;
 	int ret;
@@ -499,7 +498,7 @@ static int list_recurs_directory(direntry_t *entry UNUSEDP,
 	/* then list subdirectories */
 	subMp = *mp;
 	subMp.lookupflags = ACCEPT_DIR | NO_DOTS | NO_MSG | DO_OPEN;
-	return ret | mp->loop(mp->File, &subMp, "*");
+	return ret | mp->loop(mp->File, &subMp, "*") | GOT_ONE;
 }
 
 #if 0
@@ -507,7 +506,7 @@ static int test_directory(direntry_t *entry, MainParam_t *mp)
 {
 	Stream_t *File=mp->File;
 	Stream_t *Target;
-	char errmsg[80];
+	char errmsg[200];
 
 	if ((Target = SimpleFileOpen(0, 0, "-",
 				     O_WRONLY,
@@ -538,8 +537,7 @@ void mdir(int argc, char **argv, int type UNUSEDP)
 	int ret;
 	MainParam_t mp;
 	int c;
-	const char *fakedArgv[] = { "." };
-
+	
 	concise = 0;
 	recursive = 0;
 	wide = all = 0;
@@ -584,13 +582,6 @@ void mdir(int argc, char **argv, int type UNUSEDP)
 		}
 	}
 
-	/* fake an argument */
-	if (optind == argc) {
-		argv = (char **)fakedArgv;
-		argc = 1;
-		optind = 0;
-	}
-
 	init_mp(&mp);
 	currentDrive = '\0';
 	currentDir = 0;
@@ -602,9 +593,10 @@ void mdir(int argc, char **argv, int type UNUSEDP)
 		mp.dirCallback = test_directory;
 	} else
 #endif
-		if(recursive) {
-		mp.lookupflags = ACCEPT_DIR | DO_OPEN_DIRS | NO_DOTS;
+	if(recursive) {
+		mp.lookupflags = ACCEPT_DIR | ACCEPT_PLAIN | DO_OPEN_DIRS | NO_DOTS;
 		mp.dirCallback = list_recurs_directory;
+		mp.callback = list_file;
 	} else {
 		mp.lookupflags = ACCEPT_DIR | ACCEPT_PLAIN | DO_OPEN_DIRS;
 		mp.dirCallback = list_non_recurs_directory;
